
import java.util.*;
import java.io.*;

public class zoninghouses_font
{
   public static void main(String[] args) throws Exception
   {
      PrintWriter out = new PrintWriter(System.out);
      new zoninghouses_font(new FastScanner(System.in), out);
      out.close();
   }

   public zoninghouses_font(FastScanner in, PrintWriter out)
   {
      int N = in.nextInt();
      int Q = in.nextInt();

      int[] xs = new int[N];
      int[] ys = new int[N];
      for (int i=0; i<N; i++)
      {
         xs[i] = in.nextInt();
         ys[i] = in.nextInt();
      }

      RMQ rmqx = new RMQ(xs);
      RMQ rmqy = new RMQ(ys);
      while (Q-->0)
      {
         int a = in.nextInt()-1;
         int b = in.nextInt()-1;
         int[] aminX = rmqx.minQuery(a, b);
         int[] amaxX = rmqx.maxQuery(a, b);
         int[] aminY = rmqy.minQuery(a, b);
         int[] amaxY = rmqy.maxQuery(a, b);
      
         TreeSet<Integer> candidates = new TreeSet<>();
         candidates.add(aminX[0]); candidates.add(aminX[1]);
         candidates.add(amaxX[0]); candidates.add(amaxX[1]);
         candidates.add(aminY[0]); candidates.add(aminY[1]);
         candidates.add(amaxY[0]); candidates.add(amaxY[1]);
         int[] vs = new int[candidates.size()];
         int ptr = 0;
         for (int v : candidates)
            vs[ptr++] = v;
      
         int best = Integer.MAX_VALUE;
         for (int i=0; i<vs.length; i++)
         {
            int minX = Integer.MAX_VALUE;
            int minY = Integer.MAX_VALUE;
            int maxX = Integer.MIN_VALUE;
            int maxY = Integer.MIN_VALUE;
            for (int j=0; j<vs.length; j++)
            {
               if (i != j)
               {
                  int x = vs[i];
                  int y = vs[j];
                  minX = Math.min(minX, xs[y]);
                  minY = Math.min(minY, ys[y]);
                  maxX = Math.max(maxX, xs[y]);
                  maxY = Math.max(maxY, ys[y]); 
               }
            }
            int curX = maxX - minX;
            int curY = maxY - minY;
            int cur = Math.max(curX, curY);
            best = Math.min(best, cur);
         }
         out.println(best);
      }
   }
}

// This RMQ gives you the index of the two smallest and two largest
// values that it possesses (passed to you by an array)
class RMQ
{
   int depth, n;
   int[] vMin, vMax;
   int[][][] minValue;
   int[][][] maxValue;

   RMQ(int[] vs)
   {
      n = vs.length; 
      vMin = new int[n+1];
      vMax = new int[n+1];
      vMin[n] = Integer.MAX_VALUE;
      vMax[n] = Integer.MIN_VALUE;
      for (int i=0; i<n; i++)
         vMin[i] = vMax[i] = vs[i];
      depth = Integer.numberOfTrailingZeros(Integer.highestOneBit(n));
      minValue = new int[depth+1][n][2];
      for (int i=0; i<n; i++)
      {
         minValue[0][i][0] = i;
         minValue[0][i][1] = n;
      }
      for (int d=1; d<=depth; d++)
         for (int i=0; i+(1<<d)<=n; i++)
            mergeMin(minValue[d-1][i], minValue[d-1][i+(1<<(d-1))], minValue[d][i]);
      maxValue = new int[depth+1][n][2];
      for (int i=0; i<n; i++)
      {
         maxValue[0][i][0] = i;
         maxValue[0][i][1] = n;
      }
      for (int d=1; d<=depth; d++)
         for (int i=0; i+(1<<d)<=n; i++)
            mergeMax(maxValue[d-1][i], maxValue[d-1][i+(1<<(d-1))], maxValue[d][i]);
   }

   void mergeMin(int[] a, int[] b, int[] res)
   {
      int i = 0, j = 0;
      if (a[i] == b[j])
         i++;
      if (vMin[a[i]] < vMin[b[j]])
      {
         res[0] = a[i];
         i = 1;
      }
      else
      {
         res[0] = b[j];
         j = 1;
      }
     
      if (vMin[a[i]] < vMin[b[j]])
         res[1] = a[i];
      else
         res[1] = b[j];
   }

   int[] minQuery(int i, int j)
   {
      int d = Integer.numberOfTrailingZeros(Integer.highestOneBit(j-i+1));

      int[] left = minValue[d][i];
      int[] right = minValue[d][1+j-(1<<d)];
      int[] res = new int[2];
      mergeMin(left, right, res);
      return res;
   }

   void mergeMax(int[] a, int[] b, int[] res)
   {
      int i = 0, j = 0;
      if (a[i] == b[j])
         i++;
      if (vMax[a[0]] > vMax[b[0]])
      {
         res[0] = a[0];
         i = 1;
      }
      else
      {
         res[0] = b[0];
         j = 1;
      }
     
      if (vMax[a[i]] > vMax[b[j]])
         res[1] = a[i];
      else
         res[1] = b[j];
   }

   int[] maxQuery(int i, int j)
   {
      int d = Integer.numberOfTrailingZeros(Integer.highestOneBit(j-i+1));

      int[] left = maxValue[d][i];
      int[] right = maxValue[d][1+j-(1<<d)];
      int[] res = new int[2];
      mergeMax(left, right, res);
      return res;
   }
}

class FastScanner{
    private InputStream stream;
    private byte[] buf = new byte[1024];
    private int curChar;
    private int numChars;
    
    public FastScanner(InputStream stream)
    {
        this.stream = stream;
    }
    
    int read()
    {
        if (numChars == -1)
            throw new InputMismatchException();
        if (curChar >= numChars){
            curChar = 0;
            try{
                numChars = stream.read(buf);
            } catch (IOException e) {
                throw new InputMismatchException();
            }
            if (numChars <= 0)
                return -1;
        }
        return buf[curChar++];
    }
    
    boolean isSpaceChar(int c)
    {
        return c==' '||c=='\n'||c=='\r'||c=='\t'||c==-1;
    }
    
    boolean isEndline(int c)
    {
        return c=='\n'||c=='\r'||c==-1;
    }
    
    int nextInt()
    {
        return Integer.parseInt(next());
    }
    
    long nextLong()
    {
        return Long.parseLong(next());
    }
    
    double nextDouble()
    {
        return Double.parseDouble(next());
    }
    
    String next(){
        int c = read();
        while (isSpaceChar(c))
            c = read();
        StringBuilder res = new StringBuilder();
        do{
            res.appendCodePoint(c);
            c = read();
        }while(!isSpaceChar(c));
        return res.toString();
    }
    
    String nextLine(){
        int c = read();
        while (isEndline(c))
            c = read();
        StringBuilder res = new StringBuilder();
        do{
            res.appendCodePoint(c);
            c = read();
        }while(!isEndline(c));
        return res.toString();
    }
}
